#
#  Representation of a rule
#
#  {
#     in => inputSuffix,                -- for pattern rules
#     out => outputSuffix,              -- for pattern rules
#     depend => [...],                  -- for regular rules
#     target => [...],                  -- for regular rules
#     commands => [...]                 -- for both types
#     used => bool
#  }
#
package Rule;
use Util ('util_add_uniquely', 'util_clone');
use strict;

sub newPatternRule {
    my ($proto, $in, $out) = @_;
    bless ({ in => $in,
             out => $out,
             commands => [],
             used => 0 }, ref($proto) || $proto);
}

sub newRule {
    my $proto = shift;
    bless ({ depend => [],
             target => [],
             commands => [],
             used => 0 }, ref($proto) || $proto);
}

sub addCommand {
    my $self = shift;
    push @{$self->{commands}}, @_
}

sub addDepend {
    my $self = shift;
    util_add_uniquely(@{$self->{depend}}, @_);
}

sub addTarget {
    my $self = shift;
    util_add_uniquely(@{$self->{target}}, @_);
}

sub getDepend {
    shift->{depend}
}

sub getCommands {
    shift->{commands}
}

sub getInputSuffix {
    shift->{in}
}

sub getOutputSuffix {
    shift->{out}
}

sub clone {
    my $self = shift;
    if (exists ($self->{in})) {
        # Pattern rule
        bless { in => $self->{in},
                out => $self->{out},
                commands => util_clone($self->{commands}),
                used => 0 }, 'Rule';
    } else {
        # Regular rule
        bless { depend => util_clone($self->{depend}),
                target => util_clone($self->{target}),
                commands => util_clone($self->{commands}),
                used => 0 }, 'Rule';
    }
}

sub markUsed {
    shift->{used} = 1;
}

1;
